/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package cronjob

import (
	batchv1beta1 "k8s.io/api/batch/v1beta1"

	"k8s.libre.sh/objects/job"
)

type Mutate interface {
	GetSuccessfulJobsHistoryLimit() *int32
	GetConcurrencyPolicy() batchv1beta1.ConcurrencyPolicy
	GetStartingDeadlineSeconds() *int64
	GetSchedule() string
	GetFailedJobsHistoryLimit() *int32
	job.Mutate
}

func MutateCronJob(i Mutate, obj *batchv1beta1.CronJob) error {

	obj.Spec.SuccessfulJobsHistoryLimit = i.GetSuccessfulJobsHistoryLimit()
	obj.Spec.ConcurrencyPolicy = i.GetConcurrencyPolicy()
	obj.Spec.StartingDeadlineSeconds = i.GetStartingDeadlineSeconds()
	obj.Spec.FailedJobsHistoryLimit = i.GetFailedJobsHistoryLimit()
	obj.Spec.Schedule = i.GetSchedule()

	err := job.MutateJob(i, &obj.Spec.JobTemplate.Spec)
	if err != nil {
		return err
	}

	return nil
}
